import MCF_TypeFactory from './types/typeFactory'



export default class MCF_DefinedObject {
  constructor(definition) {
    this.definition = definition
    this.validations = {}
    this.fields = MCF_TypeFactory.buildDefinedObject(this)
    this.validateValue(definition.values)
    this.setValue(definition.values)
  }

  getValue() {
    const value = {}
    for (const field in this.fields) {
      value[field] = this.fields[field].value
    }
    return value
  }

  setValue(value) {
    for (const valueKey in value) {
      if (value[valueKey] === undefined) {
        this.fields[valueKey].value = undefined
      } else {
        if (this.fields[valueKey] === undefined) {
          throw new Error(`The field ${valueKey} doesn't exist in the current DefinedObject: ${JSON.stringify(Object.keys(this.fields), null, 2)}`)
        }
        this.fields[valueKey].setValue(value[valueKey])
      }
    }
  }

  getDefinition() {
    return this.definition
  }

  validateValue(value) {
    for (const field in this.definition.fields) {
      if (value[field] === undefined || value === null) {
        if (this.definition.fields[field].required === true) {
          throw new Error(`The field ${field} is required`)
        }
        if (this.definition.fields[field].defaultValue !== undefined) {
          value[field] = this.definition.fields[field].defaultValue
        }
        if (this.definition.fields[field].null === false) {
          throw new Error(`The field ${field} can't be undefined`)
        }
      }
    }
  }

  updateArrayField(field, arrayItem, mustBeIncluded) {
    const fieldArray = this.getValue()[field] || []

    if (!fieldArray.includes(arrayItem) && mustBeIncluded) fieldArray.push(arrayItem)
    if (fieldArray.includes(arrayItem) && !mustBeIncluded) fieldArray.splice(fieldArray.indexOf(arrayItem), 1)

    const newValue = {}
    newValue[field] = fieldArray
    this.setValue(newValue)
  }
}
